# coding=utf-8
import logging

from django.conf import settings
from django.db import models
from django.template import Engine
from django.utils.translation import ugettext_lazy as _

from emailtemplates.helpers import TemplateSourceLoader, mass_mailing_recipients

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime

    now = datetime.now

logger = logging.getLogger(__name__)


class EmailTemplate(models.Model):
    """
    Model to store email template.
    """
    title = models.CharField(_(u'template'), max_length=255)
    subject = models.CharField(_(u'subject'), max_length=255, blank=True)
    content = models.TextField(_(u'content'))
    language = models.CharField(_(u'language'), max_length=10, choices=settings.LANGUAGES,
                                default=settings.LANGUAGE_CODE)
    created = models.DateTimeField(default=now)
    modified = models.DateTimeField(default=now)

    class Meta:
        unique_together = ((u'title', u'language'),)

    def __unicode__(self):
        return u'%s -> %s' % (self.title, self.language)

    def get_default_content(self):
        engine = Engine.get_default()
        loader = TemplateSourceLoader(engine)
        try:
            return loader.get_source(self.title)
        except Exception as e:
            logger.error('Error loading template %s. Details: %s ', self.title, e)
            return ''

    def save(self, *args, **kwargs):
        if not self.content:
            self.content = self.get_default_content()
        super(EmailTemplate, self).save(*args, **kwargs)


class MassEmailMessage(models.Model):
    subject = models.CharField(_(u'subject'), max_length=255)
    content = models.TextField(_(u'content'))
    date_sent = models.DateTimeField(_(u'sent'), null=True, blank=True)

    def __unicode__(self):
        return self.subject

    @property
    def sent(self):
        return bool(self.date_sent)

    def send(self, recipients=None, force=False):
        from emailtemplates.email import EmailFromTemplate
        recipients = recipients or mass_mailing_recipients()
        if self.sent and not force:
            return False
        eft = EmailFromTemplate(
            name="emailtemplates/mass_email.html", subject=self.subject,
            template_object=self, registry_validation=False,
        )
        attachment_paths = [attachment.attachment_file.path for attachment in self.attachments.all()]
        sent_count = 0
        for recipient in recipients:
            sent = eft.send(to=[recipient], attachment_paths=attachment_paths)
            if sent:
                sent_count += 1
                logger.info(u"Successfully sent mass email message to user %s", recipient)
            else:
                logger.warning(u"Error sending mass email message to user %s", recipient)
        self.date_sent = now()
        self.save()
        return sent_count == len(recipients)


class MassEmailAttachment(models.Model):
    attachment_file = models.FileField(_(u"Attachment file"))
    mass_email_message = models.ForeignKey(MassEmailMessage, related_name="attachments", on_delete=models.CASCADE)



# coding=utf-8
import mock
import os

from django.conf import settings
from django.core import mail
from django.test import TestCase
from django.utils.html import escape
from mock import Mock

from ..email import EmailFromTemplate
from ..email import logger as email_logger
from ..helpers import substr
from ..models import EmailTemplate
from ..registry import email_templates, NotRegistered, EmailTemplateRegistry


class CheckEmail(TestCase):
    def check_email_was_sent(self, eft, to):
        self.assertTrue(len(mail.outbox) > 0)
        msg = mail.outbox[0]
        self.assertTrue(settings.DEFAULT_FROM_EMAIL in msg.from_email)
        self.assertEqual(msg.content_subtype, 'html')
        self.assertEqual(msg.subject, eft.subject)
        self.assertEqual(msg.body, eft.message)
        self.assertTrue('Message-Id' in msg.message())
        self.assertEqual(msg.to, to)


class EmailFromTemplateTest(CheckEmail):
    def setUp(self):
        mail.outbox = []
        email_templates = EmailTemplateRegistry()
        email_logger.warning = Mock()
        self.attachment_filepath = os.path.join(os.path.dirname(__file__), 'data', 'example_file.txt')

    def test_empty_object(self):
        eft = EmailFromTemplate(registry_validation=False)
        self.assertTrue(isinstance(eft, object))
        eft.render_message()
        to = ['to@example.com']
        eft.send_email(to)
        self.check_email_was_sent(eft, to)

    def test_with_empty_db_object(self):
        eft = EmailFromTemplate(registry_validation=False)
        eft.get_object()
        email_logger.warning.assert_has_calls(substr("Can't find EmailTemplate object in database"))
        email_logger.warning.assert_has_calls(substr("template in the filesystem, will use very default one"))
        eft.render_message()
        to = ['to@example.com']
        eft.send_email(to)
        self.check_email_was_sent(eft, to)

    def test_init_check_email_templates_registry(self):
        with self.assertRaises(NotRegistered):
            email_template = EmailFromTemplate("some_template.html")
        email_templates.register("some_template.html")
        email_template = EmailFromTemplate("some_template.html")
        self.assertTrue(email_templates.is_registered("some_template.html"))

    def test_send_attachment_paths(self):
        eft = EmailFromTemplate(registry_validation=False)
        to = ['to@example.com']
        eft.send(to, attachment_paths=[self.attachment_filepath])
        self.check_email_was_sent(eft, to)
        self.assertEqual(
            mail.outbox[0].attachments, 
            [('example_file.txt', u'Some content of example file.', 'text/plain')],
        )


class EmailFromTemplateWithFixturesTest(CheckEmail):
    def setUp(self):
        self.language = 'pl'
        email_templates = EmailTemplateRegistry()
        self.support_template = EmailTemplate.objects.create(
            language=self.language,
            title='support_respond.html',
            subject="Test",
            content="Support: {{ user_name }}"
        )
        mail.outbox = []
        email_logger.debug = Mock()

    def test_support_database_template(self):
        eft = EmailFromTemplate(name='support_respond.html', language=self.language, registry_validation=False)
        eft.get_object()
        email_logger.debug.assert_called_with(substr("Got template"))
        self.assertEqual(eft.template_source, 'database')
        eft.render_message()
        to = ['tester1@example.com', 'tester2@example.com']
        eft.send_email(to)
        self.check_email_was_sent(eft, to)

    def test_support_database_template_without_title(self):
        self.support_template.subject = ''
        self.support_template.save(update_fields=['subject'])
        eft = EmailFromTemplate(
            name='support_respond.html',
            subject='default email title',
            language=self.language,
            registry_validation=False
        )
        eft.get_object()
        self.assertEqual(eft.subject, 'default email title')

    def test_friends_invitation_no_database_or_filesystem_template(self):
        eft = EmailFromTemplate(registry_validation=False)
        eft.context = {'user_name': 'Alibaba',
                       'personal_message': "I'd like you te be site member!",
                       'landing_url': 'http://example.com/followers/612/'}
        eft.template = "{{ user_name }}, {{ personal_message }} {{ landing_url }}"
        eft.render_message()
        self.assertEqual(eft.message,
                         escape(u"Alibaba, I'd like you te be site member! http://example.com/followers/612/"))
        to = ['tester@example.com']
        self.assertEqual(eft.template_source, 'default')
        eft.send_email(to)
        self.check_email_was_sent(eft, to)



# coding=utf-8
from django.test import TestCase

from ..registry import EmailTemplateRegistry, RegistrationItem, HelpContext


class HelpContextTest(TestCase):

    def test_get_help_keys(self):
        help_context = HelpContext({
            'username': (u'Name of user in hello expression', u'superman_90'),
            'full_name': (u'Full user name', u'John Smith'),
            'property': u'Some other property',
        })
        self.assertDictEqual(help_context.get_help_keys(), {
            'username': u'Name of user in hello expression',
            'full_name': u'Full user name',
            'property': u'Some other property',
        })

    def test_get_help_values(self):
        help_context = HelpContext({
            'username': (u'Name of user in hello expression', u'superman_90'),
            'full_name': (u'Full user name', u'John Smith'),
            'property': u'Some other property',
        })
        self.assertDictEqual(help_context.get_help_values(), {
            'username': u'superman_90',
            'full_name': u'John Smith',
            'property': u'<property>',
        })


class RegistrationItemTest(TestCase):

    def test_context_description(self):
        item = RegistrationItem('hello_template.html', help_text=u'Hello template',
                                help_context={'username': u'Name of user in hello expression'})
        self.assertIn(u"<b>{{ username }}</b>", item.context_description())

    def test_as_form_help_text(self):
        item = RegistrationItem('hello_template.html', help_text=u'Hello template',
                                help_context={'username': u'Name of user in hello expression'})
        self.assertEqual(unicode, type(item.as_form_help_text()))
        self.assertIn("USAGE", item.as_form_help_text())
        self.assertIn("CONTEXT", item.as_form_help_text())

    def test_as_form_choice(self):
        item = RegistrationItem('hello_template.html', help_text=u'Hello template',
                                help_context={'username': u'Name of user in hello expression'})
        self.assertEqual(tuple, type(item.as_form_choice()))

    def test_safe_defaults(self):
        item = RegistrationItem('hello_template.html')
        self.assertEqual(unicode, type(item.help_text))
        self.assertEqual(dict, type(item.help_context))
        self.assertEqual(tuple, type(item.as_form_choice()))


class EmailTemplateRegistryTest(TestCase):

    def test_is_registered(self):
        registry = EmailTemplateRegistry()
        registry.register('hello_template.html')
        self.assertTrue(registry.is_registered('hello_template.html'))

    def test_get_help_text(self):
        template_registry = EmailTemplateRegistry()
        template_registry.register('hello_template.html', help_text=u'Hello template',
                                   help_context={'username': u'Name of user in hello expression'})
        help_text = template_registry.get_help_text('hello_template.html')
        self.assertEqual(help_text, u'Hello template')

    def test_get_help_context(self):
        template_registry = EmailTemplateRegistry()
        template_registry.register('hello_template.html', help_text=u'Hello template',
                                   help_context={'username': u'Name of user in hello expression'})
        help_context = template_registry.get_help_context('hello_template.html')
        self.assertIn('username', help_context)

    def test_get_help_content(self):
        template_registry = EmailTemplateRegistry()
        template_registry.register('hello_template.html', help_text=u'Hello template',
                                   help_context={
                                       'username': (u'Name of user in hello expression', u'superman_90'),
                                       'full_name': (u'Full user name', u'John Smith'),
                                       'property': u'Some other property',
                                   })
        help_content = template_registry.get_help_content('hello_template.html')
        self.assertDictEqual(help_content, {
            'username': u'superman_90',
            'full_name': u'John Smith',
            'property': u'<property>',
        })

    def test_get_email_templates(self):
        template_registry = EmailTemplateRegistry()
        template_registry.register('hello_template.html', help_text=u'Hello template',
                                   help_context={'username': u'Name of user in hello expression'})
        template_registry.register('simple_template.html', help_text=u'Simple template')
        self.assertEqual(2, len(template_registry.email_template_choices()))

    def test_email_template_choices(self):
        template_registry = EmailTemplateRegistry()
        template_registry.register('hello_template.html', help_text=u'Hello template',
                                   help_context={'username': u'Name of user in hello expression'})
        self.assertEqual(1, len(template_registry.email_template_choices()))
        template, _ = template_registry.email_template_choices()[0]
        self.assertEqual('hello_template.html', template)

    def test_registration_items(self):
        template_registry = EmailTemplateRegistry()
        template_registry.register('hello_template.html', help_text=u'Hello template',
                                   help_context={'username': u'Name of user in hello expression'})
        items = template_registry.registration_items()
        self.assertEqual(1, len(items))
        self.assertEqual('hello_template.html', items[0].path)







# encoding: utf-8
from __future__ import unicode_literals
import mock
import os

from django.core import mail
from django.core.files import File
from django.test import TestCase

from emailtemplates.helpers import TemplateSourceLoader
from emailtemplates.models import EmailTemplate, MassEmailMessage, MassEmailAttachment


class EmailTemplateTest(TestCase):
    def setUp(self):
        self.default_content = '<h1>TEST DEFAULT CONTENT</h1>'
        self.email_template = EmailTemplate.objects.create(title='template-1.html')

    @mock.patch.object(TemplateSourceLoader, 'get_source')
    def test_get_default_content(self, mock_source):
        mock_source.return_value = self.default_content
        self.assertEqual(self.email_template.get_default_content(), self.default_content)

    @mock.patch.object(TemplateSourceLoader, 'get_source', mock.Mock(side_effect=Exception('error...')))
    def test_get_empty_default_content_if_error(self):
        self.assertEqual(self.email_template.get_default_content(), '')

    @mock.patch.object(TemplateSourceLoader, 'get_source')
    def test_save_default_content(self, mock_source):
        mock_source.return_value = self.default_content
        email_template = EmailTemplate.objects.create(title='template-2.html')
        self.assertEqual(email_template.content, self.default_content)

    @mock.patch.object(TemplateSourceLoader, 'get_source')
    def test_do_not_override_existing_content(self, mock_source):
        mock_source.return_value = self.default_content
        email_template = EmailTemplate.objects.create(title='template-2.html', content='<h1>New content</h1>')
        self.assertEqual(email_template.content, '<h1>New content</h1>')


class MassEmailMessageTest(TestCase):
    def setUp(self):
        self.mass_email_message = MassEmailMessage.objects.create(
            subject="Temat maila",
            content="<p>Treść emaila</p>"
        )
        self.attachment_filepath = os.path.join(os.path.dirname(__file__), 'data', 'example_file.txt')

    def test_send(self):
        recipients = ["person@example.com"]
        sent = self.mass_email_message.send(recipients)
        self.assertTrue(sent)
        self.assertTrue(self.mass_email_message.sent)
        self.assertEqual(mail.outbox[0].to, recipients)
        self.assertEqual(mail.outbox[0].subject, "Temat maila")
        self.assertEqual(mail.outbox[0].body, "<p>Treść emaila</p>")

    def test_send_with_attachments(self):
        attachment = MassEmailAttachment.objects.create(
            attachment_file=File(open(self.attachment_filepath, 'r'), 'example_file.txt'), 
            mass_email_message=self.mass_email_message,
        )
        recipients = ["person@example.com"]
        sent = self.mass_email_message.send(recipients)
        self.assertTrue(sent)
        self.assertEqual(
            mail.outbox[0].attachments, 
            [('example_file.txt', u'Some content of example file.', 'text/plain')],
        )


# coding=utf-8
from django.contrib import admin
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from .forms import EmailTemplateAdminForm, MassEmailMessageForm, MassEmailAttachmentForm
from .models import EmailTemplate, MassEmailMessage, MassEmailAttachment


class EmailTemplateAdmin(admin.ModelAdmin):
    """
    Admin view of EmailTemplate
    """
    list_display = ('title', 'language', 'subject',)
    list_display_links = ('title',)
    list_filter = ('title', 'language',)
    search_fields = ('title', 'subject')
    form = EmailTemplateAdminForm
    save_on_top = True
    save_as = True
    readonly_fields = ['show_links', 'created', 'modified']

    def show_links(self, obj):
        if not obj.pk:
            return ''
        return u'<a href="%s" target="_blank">%s</a>' % (
            reverse('email_preview', kwargs={'pk': obj.pk}), _('Show email preview')
        )

    show_links.allow_tags = True
    show_links.short_description = _('Actions')


admin.site.register(EmailTemplate, EmailTemplateAdmin)


class MassEmailAttachmentInline(admin.TabularInline):
    model = MassEmailAttachment
    form = MassEmailAttachmentForm


class MassEmailMessageAdmin(admin.ModelAdmin):
    list_display = ('subject', 'date_sent')
    readonly_fields = ['date_sent']
    form = MassEmailMessageForm
    inlines = [MassEmailAttachmentInline]


admin.site.register(MassEmailMessage, MassEmailMessageAdmin)
